/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <stdio.h>

#include "tray.h"
#include "intl.h"

#define debug_printf printf

#define SYSTRAY_HINT _("GtkTray")
#define WM_TRAYMESSAGE WM_USER

enum _SYSTRAY_CMND {
	SYSTRAY_CMND_MENU_EXIT,
	SYSTRAY_CMND_HIDE,
	SYSTRAY_CMND_SHOW
};
typedef enum _SYSTRAY_CMND SYSTRAY_CMND;

static GtkWidget *MainWin;
static HWND mainwin_hwnd;
static HINSTANCE this_hinstance;

static HWND systray_hwnd=0;
static HICON systray_icon=0;
static NOTIFYICONDATA sysicon_nid;
static HMENU systray_menu=0;

static long oldwndproc;

static void MinimizeToTray(HWND hWnd);
static void RestoreFromTray(HWND hWnd);

static void systray_show_menu(int x, int y) {
	SetForegroundWindow(systray_hwnd);
	TrackPopupMenu(systray_menu,         // handle to shortcut menu
		       TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_LEFTBUTTON,
		       x,                   // horizontal position, in screen coordinates
		       y,                   // vertical position, in screen coordinates
		       0,                   // reserved, must be zero
		       systray_hwnd,        // handle to owner window
		       NULL                 // ignored
		       );
}

static void systray_create_menu(void) {
	if((systray_menu = CreatePopupMenu())) {
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_SHOW, _("Show")))
			debug_printf("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_HIDE, _("Hide")))
			debug_printf("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_SEPARATOR, 0, 0))
			debug_printf("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_MENU_EXIT, _("Exit")))
			debug_printf("AppendMenu error: %d\n", GetLastError());
	} else
		debug_printf("CreatePopupMenu error: %d\n", GetLastError());
}

static void systray_init_icon(HWND hWnd, HICON icon) {
	ZeroMemory(&sysicon_nid,sizeof(sysicon_nid));
	sysicon_nid.cbSize=sizeof(NOTIFYICONDATA);
	sysicon_nid.hWnd=hWnd;
	sysicon_nid.uFlags=NIF_ICON | NIF_MESSAGE | NIF_TIP;
	sysicon_nid.uCallbackMessage=WM_TRAYMESSAGE;
	sysicon_nid.hIcon=icon;
	strcpy(sysicon_nid.szTip,SYSTRAY_HINT);
}

static LRESULT CALLBACK systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) {

	switch(msg) {
	case WM_COMMAND:
		debug_printf("WM_COMMAND\n");
		switch(LOWORD(wparam)) {
		case SYSTRAY_CMND_MENU_EXIT:
			//XXX app_will_exit(NULL, MainWin);
			break;
		case SYSTRAY_CMND_SHOW:
			debug_printf("Show\n");
			RestoreFromTray(mainwin_hwnd);
			break;
		case SYSTRAY_CMND_HIDE:
			MinimizeToTray(mainwin_hwnd);
			debug_printf("Hide\n");
			break;
		}
		break;
	case WM_TRAYMESSAGE:
	{
		if( lparam == WM_LBUTTONDBLCLK ) {
			
			if (IsWindowVisible(mainwin_hwnd))
				MinimizeToTray(mainwin_hwnd);
			else
				RestoreFromTray(mainwin_hwnd);
			debug_printf("Systray got double click\n");
		}
		if( lparam == WM_RBUTTONUP ) {
			POINT mpoint;
			GetCursorPos(&mpoint);
			systray_show_menu(mpoint.x, mpoint.y);
		}
		break;
	}
	default: ;
	}/* end switch */

	return DefWindowProc(hwnd, msg, wparam, lparam);
}

static HWND systray_create_hiddenwin() {
	WNDCLASS wc;
	TCHAR wname[32];

	ZeroMemory(&wc, sizeof(wc));
	strcpy(wname, "HiddenGtkTrayWin78902134567890");
	wc.lpfnWndProc	 = (WNDPROC)systray_mainmsg_handler;
	wc.hInstance	 = this_hinstance;
	wc.lpszClassName = wname;
	RegisterClass(&wc);
	return (CreateWindow(wname, "", 0, 0, 0, 0, 0, 
		GetDesktopWindow(), NULL, this_hinstance, 0));
}

extern void SetWindowState(gint winstate) {
	switch (winstate) {
	case 0: break;
	case 1: MinimizeToTray(mainwin_hwnd); break;//ShowWindow(mainwin_hwnd, SW_MINIMIZE); break;
	case 2: ShowWindow(mainwin_hwnd, SW_MAXIMIZE); break;
	}
}

extern void ShowTrayIcon(boolean show) {
	if (show)
		Shell_NotifyIcon(NIM_ADD,&sysicon_nid);
	else
		Shell_NotifyIcon(NIM_DELETE,&sysicon_nid);
}

static void MinimizeToTray(HWND hWnd) {
	if (trayplugin_enable)
		ShowWindow(hWnd,SW_HIDE);
	else
		ShowWindow(hWnd,SW_MINIMIZE);
	ShowTrayIcon(trayplugin_enable);
}

static void RestoreFromTray(HWND hWnd) {
	ShowWindow(hWnd,SW_SHOW);
	SetActiveWindow(hWnd);
	SetForegroundWindow(hWnd);
	if (!trayplugin_persistent)
		ShowTrayIcon(FALSE);
}

static LRESULT CALLBACK mainwndproc_handler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	boolean handled=TRUE;

	switch (uMsg) {
		case WM_SYSCOMMAND: {
			switch (wParam) {
				case SC_MINIMIZE: MinimizeToTray(mainwin_hwnd);	break;
				default: handled=FALSE;
			}
		} ; break;
		case WM_CLOSE: tray_close(); handled=FALSE; break;
		default: handled=FALSE;
	}

	if (handled)
		return 0;
	else
		return CallWindowProc((WNDPROC)oldwndproc, hWnd, uMsg, wParam, lParam);
}

extern int tray_close() {
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, oldwndproc);
	ShowTrayIcon(FALSE);
	DestroyWindow(systray_hwnd);
	return 1;
}

extern int tray_init(GtkWidget *mainwin) {
	MainWin = mainwin;
	this_hinstance = GetModuleHandle(NULL);
	mainwin_hwnd=GDK_DRAWABLE_XID(mainwin->window);
	oldwndproc = GetWindowLong(mainwin_hwnd, GWL_WNDPROC);
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, (long)mainwndproc_handler);
	systray_hwnd = systray_create_hiddenwin();
	systray_create_menu();
	systray_icon = LoadIcon(this_hinstance,MAKEINTRESOURCE(IDI_ICON1));
	systray_init_icon(systray_hwnd, systray_icon);
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
	return 1;
}
