/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif
#include "defs.h"

#include "plugin.h"
#include "prefs_gtk.h"
#include "intl.h"
#include "common/utils.h"

#include "tray.h"

/* ------------------------------------------------------------------------- */

static gint findmain_timeout_tag=0;
static struct TrayPluginPage *trayplugin_page;

static PrefParam param[] = {
	{"enable", "TRUE", &trayplugin_enable, P_BOOL,
	 NULL, NULL, NULL},
	{"persistent", "TRUE", &trayplugin_persistent, P_BOOL,
	 NULL, NULL, NULL},
	{"windowstate", "0", &trayplugin_windowstate, P_INT,
	 NULL, NULL, NULL},
	{"restore_windowstate", "TRUE", &trayplugin_restore_windowstate, P_BOOL,
	 NULL, NULL, NULL},

	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
};

struct TrayPluginPage
{
	PrefsPage page;
	
	GtkWidget *enable;
	GtkWidget *persistent;
	GtkWidget *windowstate;
	GtkWidget *restore_windowstate;
};

/* ------------------------------------------------------------------------- */

static gint findmain_timeout_cb(gpointer *data) {
	gboolean initialized;
	mainwindow = mainwindow_get_mainwindow();
	g_return_val_if_fail(mainwindow, TRUE);

	initialized = tray_init(mainwindow->window);
	SetWindowState(trayplugin_windowstate);
	return FALSE; /* remove timeout */
}

/* ------------------------------------------------------------------------- */

void trayplugin_save_config()
{
	PrefFile *pfile;
	gchar *rcpath;

	debug_print("Saving Tray plugin Page\n");

	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMON_RC, NULL);
	pfile = prefs_write_open(rcpath);
	g_free(rcpath);
	if (!pfile || (prefs_set_block_label(pfile, "TrayPlugin") < 0))
		return;

	if (prefs_write_param(param, pfile->fp) < 0) {
		g_warning("failed to write Tray plugin configuration to file\n");
		prefs_file_close_revert(pfile);
		return;
	}
	//fprintf(pfile->fp, "\n");

	prefs_file_close(pfile);
}

/* ------------------------------------------------------------------------- */

static void trayplugin_create_widget_func(PrefsPage * _page, GtkWindow *window, gpointer data)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;
	GtkWidget *vbox1;
	GtkWidget *enable;
	GtkWidget *persistent;
	GtkWidget *windowstate_adj;
	GtkWidget *windowstate;
	GtkWidget *restore_windowstate;
	GtkWidget *windowstate_desc;


	vbox1 = gtk_vbox_new(TRUE, 4);
	gtk_widget_show(vbox1);
	//gtk_container_add(GTK_CONTAINER(vbox1), vbox1);

	enable = gtk_check_button_new_with_label("enable");
	gtk_widget_show(enable);
	gtk_box_pack_start(GTK_BOX(vbox1), enable, TRUE, TRUE, 4);

	persistent = gtk_check_button_new_with_label("persistent");
	gtk_widget_show(persistent);
	gtk_box_pack_start(GTK_BOX(vbox1), persistent, TRUE, TRUE, 4);

	windowstate_desc = gtk_label_new(_("windowstate (0:don't set 1:minimized 2:maximized"));
	gtk_widget_show(windowstate_desc);
	gtk_box_pack_start(GTK_BOX(vbox1), windowstate_desc, TRUE, TRUE, 4);
	
	windowstate_adj = gtk_adjustment_new(783, 0, 2, 1, 10, 10);
	windowstate =
	    gtk_spin_button_new(GTK_ADJUSTMENT(windowstate_adj), 1, 0);
	gtk_widget_show(windowstate);
	gtk_box_pack_start(GTK_BOX(vbox1), windowstate, FALSE, TRUE, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(windowstate), TRUE);

	restore_windowstate = gtk_check_button_new_with_label("restore_windowstate");
	gtk_widget_show(restore_windowstate);
	gtk_box_pack_start(GTK_BOX(vbox1), restore_windowstate, TRUE, TRUE, 4);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable), trayplugin_enable);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(persistent), trayplugin_persistent);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(windowstate), (float) trayplugin_windowstate);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(restore_windowstate), trayplugin_restore_windowstate);

	page->enable = enable;
	page->persistent = persistent;
	page->windowstate = windowstate;
	page->restore_windowstate = restore_windowstate;

	page->page.widget = vbox1;
}

static void trayplugin_destroy_widget_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin widget\n");
}

static void trayplugin_save_func(PrefsPage *_page)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;
	gchar *tmp;
	debug_print("Saving Tray plugin Page\n");
	trayplugin_enable = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->enable));
	trayplugin_persistent = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->persistent));
	tmp = gtk_editable_get_chars(GTK_EDITABLE(page->windowstate), 0, -1);
	trayplugin_windowstate = atoi(tmp);
	g_free(tmp);
	trayplugin_restore_windowstate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->restore_windowstate));
	trayplugin_save_config();
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
}

static void trayplugin_destroy_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin Page\n");
	g_free(_page);
}

/* ------------------------------------------------------------------------- */

gint plugin_init(gchar **error)
{
	struct TrayPluginPage *page;
	trayplugin_persistent = TRUE;
	trayplugin_enable = TRUE;

	/* init config */
	prefs_set_default(param);
	prefs_read_config(param, "TrayPlugin", COMMON_RC);

	page = g_new0(struct TrayPluginPage, 1);
	page->page.path = _("Filtering/TrayPlugin");
	page->page.create_widget = trayplugin_create_widget_func;
	page->page.destroy_widget = trayplugin_destroy_widget_func;
	page->page.save_page = trayplugin_save_func;
	page->page.destroy_page = trayplugin_destroy_func;
	prefs_gtk_register_page((PrefsPage *) page);
	trayplugin_page = page;
	debug_print("Tray plugin config page created\n");

	/* wait for mainwindow to show up */
	findmain_timeout_tag = gtk_timeout_add( 100, findmain_timeout_cb, NULL );
	debug_print("Tray plugin loaded\n");
	return 0;
}

void plugin_done()
{
	gtk_timeout_remove( findmain_timeout_tag );
	tray_close();
	debug_print("Tray plugin unloaded\n");
}

const gchar *plugin_name()
{
	return "Tray plugin 1.0";
}

const gchar *plugin_desc()
{
	return "This Plugin minimizes Sylpheed to tray.\n";
}
